#Exercise 3 template for CHE210D

#import python modules
import numpy as np
import time
import math

#import compiled fortran library
import projectlib

import sys
import os



#========== GLOBAL VARIABLES ==========
#distance cutoff for pairwise interactions
Cut = 2.5
#set the random number seed; useful for debugging
np.random.seed(342324)

def InitPositions(N, L):
    """Returns an array of initial positions of each atom,
placed on a cubic lattice for convenience.
Input:
    N: number of atoms
    L: box length
Output:
    Pos: (N,3) array of positions
"""
    #make the position array
    Pos = np.zeros((N,3), float)
    #compute integer grid # of locations for cubic lattice
    NLat = int(N**(1./3.) + 1.)
    LatSpac = L / NLat
    #make an array of lattice sites
    r = LatSpac * np.arange(NLat, dtype=float) - 0.5*L
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
            for z in r:
                Pos[i] = np.array([x,y,z], float)
                #add a random offset to help initial minimization
                Offset = 0.05 * LatSpac * (np.random.rand(3) - 0.5)
                Pos[i] = Pos[i] + Offset
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos
    return Pos



Cut =2.5
T=1.0
RecordFreq=2000
delrmax=0.12
miu=0.0
L=10.0
V = L**(3.0)
stepsize=400000
REq=0.9


Pos=InitPositions (2,0.9*L)

i=0
DisplaceAttempt=0
RejCount=0

while i<=stepsize:
    DisplorAddDel=np.random.random_sample()
    if DisplorAddDel>=0.5:
        N=len(Pos)
        AddorDel=np.random.random_sample()
        if AddorDel<0.5 and N>2:                          
            DeleteAtom=np.random.randint(len(Pos))
            U1=projectlib.calcsingleatomenergy(Pos,L,DeleteAtom,Cut)
            delU=-U1
            Pacc=np.minimum(0,(-delU/T-miu/T)+math.log(N/V))
            r=np.random.random()
            if Pacc>math.log(r):
                Pos=np.delete(Pos,DeleteAtom-1,0)
        else: 
            OldPos=Pos.copy()
            AddPos=(L-2.0*REq)*np.random.random_sample((1,3))-(L/2.0-REq)
            Pos=np.vstack((Pos,AddPos))
            U1=projectlib.calcsingleatomenergy(Pos,L,N,Cut)
            delU=U1
            Pacc=np.minimum(0,-delU/T+miu/T+math.log(V/(N+1)))
            r=np.random.random()
            if Pacc<=math.log(r):
                Pos=OldPos
    else:
        DisplaceAttempt=DisplaceAttempt+1
        MoveAtom=np.random.randint(len(Pos))
        OldPos=Pos[MoveAtom,:].copy()
        U1=projectlib.calcsingleatomenergy(Pos,L,MoveAtom,Cut)
        delr=2*delrmax*np.random.random_sample((1,3))-delrmax
        Pos[MoveAtom,:]=OldPos+delr
        if Pos[MoveAtom,2]>=(L/2.0-REq) or Pos[MoveAtom,2]<=(-L/2.0+REq):
            Pos[MoveAtom,:]=OldPos
            RejCount=RejCount+1
        else:
            U2=projectlib.calcsingleatomenergy(Pos,L,MoveAtom,Cut)    
            delU=U2-U1
            if delU>0:
                Pacc=math.exp(-delU/T)
                m=np.random.random()
                if Pacc<=m:
                    Pos[MoveAtom,:]=OldPos
                    RejCount=RejCount+1            
    if i%RecordFreq==0:
        PEnergy=projectlib.calcenergy(Pos,L,Cut)
        print N,PEnergy,RejCount,DisplaceAttempt
    i=i+1


Zcoordinate=Pos[:,2]
Hist=np.histogram(Zcoordinate, bins=10, range=(-L/2.0+REq,L/2.0-REq))
print Hist


